Libname.a = "DEMOLIB"
if not isfile(libname.a+".lib")
   then createlib Libname.a
endif

;=============== Button Procedures ==================

;  Input: row.n and col.n = upper right corner of button position
;         color.n = color of button and label
;         label.a = label for button
;         type.a  = type of button: "F" for flush-looking, else for regular
;                   showDialog type
;
; Return: window handle for newly created button
proc c.CreateButton.h(row.n,col.n,color.n,label.a,type.a)
   private y,                ;temp dynarray for window attributes
           h,                ;temp window handle of new button
           width.n,          ;width of new button window
           origCanvas.h,     ;save current canvas
           origWin.h,        ;save current window
           hiLite.n,         ;color of highlighted border
           lowLite.n         ;color of lowlighted border

   origCanvas.h = getcanvas()
   origWin.h = getwindow()

   DynArray y[]
      y["HASSHADOW"] = FALSE
      y["HASFRAME"] = False
      y["FLOATING"] = True
      y["STYLE"] = color.n
      y["TITLE"] = label.a
      y["CANVASHEIGHT"] =   3

   if type.a = "F"
      then width.n = len(label.a)+4
           y["CANVASWIDTH"] = width.n

           Window Create
              @row.n,col.n  Height 3 Width width.n
              Attributes y
           To h

           PaintCanvas Attribute color.n 0,0,2,width.n-1

           hiLite.n = c.AttribConvert.n(color.n,TRUE)
           lowLite.n = c.AttribConvert.n(color.n,FALSE)

           Frame Single From 0,0 To 2,width.n-1
           PaintCanvas Attribute hiLite.n 0,0,0,width.n-1
           PaintCanvas Attribute lowLite.n 2,0,2,width.n-1
           PaintCanvas Attribute hiLite.n 0,0,2,0
           PaintCanvas Attribute lowLite.n 0,width.n-1,2,width.n-1

           @1,2 ?? label.a
           PaintCanvas Attribute color.n 1,2,1,2+len(label.a)

      else width.n = len(label.a)+3
           color.n = int(color.n/16)*16
           y["CANVASWIDTH"] = width.n

           Window Create
              @row.n,col.n Height 2 Width width.n
              attributes y
           to h

           Paintcanvas Attribute color.n 0,0,0,width.n-1

           @0,1 ?? label.a

           Paintcanvas attribute 32 0,0,0,width.n-2   ;button color

           Paintcanvas fill("") attribute color.n    ;shadow
              0,width.n-1,0,width.n-1

           PaintCanvas fill("") attribute color.n    ;shadow
              1,1,1,width.n-1

   endif

   setcanvas origcanvas.h
   window select origWin.h

   return h

endproc
writelib Libname.a c.CreateButton.h
release procs c.CreateButton.h
??"."

;==========

;  Input: status.a = status of button, either "UP" or "DOWN"
;         button.h = handle of button to color
; Return: nothing
proc c.ColorButton.u(status.a,button.h)
   private origCanvas.h,          ;temp window handle of current canvas
           y,                     ;temp dynarray for win attributes
           c,                     ;hold last column for coloring border
           hiLite.n,              ;color of hilighted border
           lowLite.n               ;color of lowlighted border

   origCanvas.h = getcanvas()

   setcanvas button.h

   window getattributes button.h to y
   c = y["canvaswidth"] - 1


   if y["height"] = 3          ;Flush type button
      then hiLite.n = c.AttribConvert.n(y["style"],TRUE)
           lowLite.n = c.AttribConvert.n(y["style"],FALSE)

           if status.a = "DOWN"
              then PaintCanvas Attribute lowLite.n 0,0,0,c    ;color button so
                   PaintCanvas Attribute hiLite.n  2,0,2,c    ;that it
                   PaintCanvas Attribute lowLite.n 0,0,2,0    ;looks pressed
                   PaintCanvas Attribute hiLite.n  0,c,2,c

              else PaintCanvas Attribute hiLite.n  0,0,0,c    ;color button so
                   PaintCanvas Attribute lowLite.n 2,0,2,c    ;that it
                   PaintCanvas Attribute hiLite.n  0,0,2,0    ;looks un-pressed
                   PaintCanvas Attribute lowLite.n 0,c,2,c
           endif

      else                     ;normal showDialog type button
           if status.a = "DOWN"
              then PaintCanvas fill(" ") attribute int(y["style"]/16)*16
                      0,0,1,c
                   Paintcanvas fill(" "+y["title"]+" ") attribute 32
                      0,1,0,c

              else PaintCanvas fill(" "+y["title"]+" ") attribute 32
                      0,0,0,c-1
                   Paintcanvas fill("") attribute int(y["style"]/16)*16
                      0,c,0,c
                   Paintcanvas fill("") attribute int(y["style"]/16)*16
                      1,1,1,c
           endif

   endif

   setcanvas origCanvas.h

endproc
writelib Libname.a c.ColorButton.u
release procs c.ColorButton.u
??"."

;==========

proc c.ButtonWait.u()
   private buttonUp.l

   buttonUp.l = FALSE
   c.ColorButton.u("DOWN",button.h)     ;color the button 'down'

   ;The following wait will trap all events except idle, and deny any event
   ;except mouse 'move' or mouse 'up'.  A user will not normally try to escape
   ;out of the workspace while holding down a mouse button.  These restrictions
   ;greatly simplify event handling.

   Wait workspace
      proc "c.ButtonWaitProc.n"
         key "all"
         mouse "MOVE","UP"
         message "all"
         trigger "all"
   endwait

endproc
writelib Libname.a c.ButtonWait.u
release procs c.ButtonWait.u
??"."

;==========

;  This is a specifically designed wait proc event handler to deal with
;  mouse issues
proc c.ButtonWaitProc.n(event.a,desc.y,cycle.n)
   private win.h

   switch
      case event.a = "EVENT" and desc.y["type"] = "MOUSE":

         win.h = windowAt(desc.y["row"], desc.y["col"])

         switch
              case desc.y["action"] = "MOVE":

                 if win.h <> button.h
                    then if not buttonUp.l
                            then c.ColorButton.u("UP",button.h)
                                 buttonUp.l = TRUE
                         endif

                    else if buttonUp.l
                            then c.ColorButton.u("DOWN",button.h)
                                 buttonUp.l = FALSE
                         endif
                 endif
                 return 0

              case desc.y["action"] = "UP":
                 c.ColorButton.u("UP",button.h)
                 if win.h = button.h            ;up occurred on button pressed
                    then if isassigned(buttons.y[win.h])
                            then execproc buttons.y[win.h]
                         endif
                 endif
                 return 2

           endswitch

      case event.a = "OPEN":
         return 0

      case event.a = "CLOSE":
         return 2

      otherwise: return 1

   endswitch

endproc
writelib Libname.a c.ButtonWaitproc.n
release procs c.ButtonWaitproc.n
??"."

;==========

; ============================================================================
;       TITLE: AttribConvert.n      (c) 1992, 1993 DataStar International
;     RETURNS: Color attribute
; DESCRIPTION: Returns either the intense foreground of a background color if
;              highlight.l = true, else black on background color.
; ----------------------------------------------------------------------------
PROC c.AttribConvert.n(color.n,       ; Background color
                          highlight.l)   ; True=highlight, false=lowlight

   Private  attribute.n

   IF color.n < 16           ; Special handling for black
      then attribute.n = IIF(highlight.l, 7, 8)
      else attribute.n = (Int(color.n/16)*16) + IIF(highlight.l,Int(color.n/16)+8,0)
   endif
   Return attribute.n
ENDPROC
WriteLib libName.a c.AttribConvert.n
release procs c.AttribConvert.n
??"."

;=============== End Button Procedures ==================

;=============== Common Utility Procedures ==============

;-------------------------------------------------------------------------------
;  c.WaitProc.n - Generic waitproc used throughout apps.
;
;  This procedure assumes that a procedure dictionary has been created in
;  dynarray 'dict.y'.  This dictionary is populated with procedures that will
;  be called if the event it is tagged with occurs.
;
;  This procedure will check for a tag in the dictionary labeled with the most
;  specific event description: tablename+fieldname+eventname.  If there are no
;  matches, this procedure will then look for the next most specific event
;  description: tablename+eventname.  If nothing matches, it will then look for
;  just the least specific event description: the event itself.
;
;  If a match is found, the procedure will execproc that dictionary entry.

proc c.WaitProc.n(Event.a,Desc.y,Cycle.n)
   private map.a

   if event.a = "EVENT"
      then switch
              case desc.y["type"] = "KEY":
                 switch
                    case isassigned(dict.y[table()+field()+strval(desc.y["keycode"])]):
                       map.a = table()+field()+strval(desc.y["keycode"])

                    case isassigned(dict.y[table()+strval(desc.y["keycode"])]):
                       map.a = table()+strval(desc.y["keycode"])

                    case isassigned(dict.y[desc.y["keycode"]]):
                       map.a = desc.y["keycode"]

                    case isassigned(dict.y["KEY"]):
                       map.a = "key"

                    otherwise: if isassigned(developerMode.l) and developerMode.l
                                   then message " Warning: untrapped key event ",desc.y["keycode"]
                               endif
                               return 0
                 endswitch

              case desc.y["type"] = "MESSAGE":
                 switch
                    case isassigned(dict.y[table()+field()+desc.y["message"]]):
                       map.a = table()+field()+desc.y["message"]

                    case isassigned(dict.y[table()+desc.y["message"]]):
                       map.a = table()+desc.y["message"]

                    case isassigned(dict.y[desc.y["message"]]):
                       map.a = desc.y["message"]

                    otherwise: if isassigned(developerMode.l) and developerMode.l
                                  then message " Warning: untrapped message event ",desc.y["message"]
                               endif
                               return 0
                 endswitch

              case desc.y["type"] = "MOUSE":
                 switch
                    case isassigned(dict.y[table()+field()+"MOUSE"]):
                       map.a = table()+field()+"MOUSE"

                    case isassigned(dict.y[table()+"MOUSE"]):
                       map.a = table()+"MOUSE"

                    case isassigned(dict.y["MOUSE"]):
                       map.a = "MOUSE"

                    otherwise: if isassigned(developerMode.l) and developerMode.l
                                  then message " Warning: Untrapped mouse event "
                               endif
                               return 0
                 endswitch

           endswitch

      else switch       ;triggers
              case isassigned(dict.y[table()+field()+Event.a]):
                 map.a = table()+field()+Event.a

              case isassigned(dict.y[table()+Event.a]):
                 map.a = table()+Event.a

              case isassigned(dict.y[Event.a]):
                 map.a = Event.a

              otherwise: if isassigned(developerMode.l) and developerMode.l
                            then message " Warning: untrapped trigger: ",event.a
                         endif
                         return 0
           endswitch

   endif

   execproc dict.y[map.a]
   return retval

endproc
writelib Libname.a c.WaitProc.n
release procs c.WaitProc.n
??"."

;==========


;============== End Common Utility Procedures ================

; Procedure to view a table of calendar dates, and the colors associated
; with those dates.

proc ColorCal.u()
   private y,           ;temp dynarray for window attributes
           h.y,         ;for storage of window handles
           d.y,
           dict.y,      ;stored procedure dictionary (similar to WPP)
           buttons.y    ;for storage of button handles

   echo off
   clearall

   dynarray h.y[]

   view "colorday"
   window move getwindow() to -100,-100

   formkey
   window handle form to h.y["form"]
   window getattributes h.y["form"] to y
      y["hasframe"] = false
      y["height"] = 21
      y["width"] = 31
   window setattributes h.y["form"] from y
   window move h.y["form"] to 2,25

   showpulldown
      format("w78,ac","Exit"):format("w80,ac","Editing Color Calendar"):"Exit"
   endmenu

   prompt format("w80,ac","Editing Color Calendar")

   dynarray dict.y[]
   dict.y["mouse"] = "Mouse.n"           ;define procedure for mouse events
   dict.y["key"] = "Key.n"               ;define procedure for key events
   dict.y["menuselect"] = "Menu.n"       ;define procedure for message events

   dynarray buttons.y[]
   buttons.y[c.CreateButton.h(19,30,23,"Colors","F")] = "DboxPopup.a"
   buttons.y[c.CreateButton.h(20,43,23,"Help","")] = "Help.u"

   coeditkey
   echo normal
   wait workspace
      proc "c.WaitProc.n"
      key "F2","ESC"
      mouse "DOWN"
      message "Menuselect"
      trigger "DepartRow"
   endwait

   echo off
   Menu {} {Desktop} {empty}

endproc
writelib Libname.a ColorCal.u
release procs ColorCal.u
??"."

;==========

; Procedure to handle key events
proc Key.n()

   ;only arrive here on F2 or ESC
   echo off
   if desc.y["keycode"] = -60       ;F2
      then postrecord
           if not retval
              then message " Duplicate record "
                   return 1
              else do_it!
                   return 2
           endif

      else undo
           do_it!

   endif
   return 2

endproc
writelib Libname.a Key.n
release procs Key.n
??"."

;==========

;  Procedure to handle menu events
proc Menu.n()

   echo off

   postrecord
   if not retval
      then message " Duplicate record "
           return 1
      else do_it!
           return 2
   endif

endproc
writelib Libname.a Menu.n
release procs Menu.n
??"."

;==========

;  Procedure to handle mouse events
proc Mouse.n()
   private button.h

   ;this procedure is only called on Mouse down (per waitproc specs)
   button.h = windowAt(desc.y["row"], desc.y["col"])

   if isassigned(buttons.y[button.h])           ;if mouse down on a button
      then
           c.ButtonWait.u()                     ;wait for a button action
   endif

   window select h.y["form"]                    ;return to table

   return 0

endproc
writelib Libname.a Mouse.n
release procs Mouse.n
??"."

;==========

;  Dummy procedure to display effects of pushing the help button
proc Help.u()

   Message " You pressed the help button "

endproc
writelib Libname.a Help.u
release procs Help.u
??"."

;==========

;  Dialog box to show available color choices
proc DboxPopup.a()
   private color.n, button.l, PaintPal.OriginalPalette, colors.r

   array colors.r[5]
      colors.r[1] = "Blue"
      colors.r[2] = "Green"
      colors.r[3] = "Orange"
      colors.r[4] = "Red"
      colors.r[5] = "Yellow"

   ; PaintPAL_Generated_Code_Begin(544572188)

   ; PaintPAL_Color_Setup_Begin
   GetColors To PaintPAL.OriginalPalette
   DynArray PaintPAL.ColorDeviations[]
   PaintPAL.ColorDeviations[1046] =  48  ;Cluster Normal
   PaintPAL.ColorDeviations[1047] =  63  ;Cluster Selected
   PaintPAL.ColorDeviations[1048] =  62  ;Cluster Shortcut
   SetColors From PaintPAL.ColorDeviations
   Release Vars PaintPAL.ColorDeviations
   ; PaintPAL_Color_Setup_End

   ShowDialog "Available Colors"
      @9,15 Height 11 Width 28

      RadioButtons @1,7 Height 5 Width 12
         colors.r[1],
         colors.r[2],
         colors.r[3],
         colors.r[4],
         colors.r[5]
         Tag "color.tag"
         To color.n

      PushButton @7,2 Width 10
         "~S~elect"
         Ok
         Default
         Value "TRUE"
         Tag "select.tag"
         To button.l

      PushButton @7,14 Width 10
         "~C~ancel"
         Cancel
         Value "FALSE"
         Tag "cancel.tag"
         To button.l


   EndDialog

   ; PaintPAL_Color_Cleanup_Begin
   SetColors From PaintPAL.OriginalPalette
   Release Vars PaintPAL.OriginalPalette
   ; PaintPAL_Color_Cleanup_End

   ; PaintPAL_Generated_Code_End(544572188)

   if retval
      then if field() = "COLOR"
              then [] = upper(colors.r[color.n])
           endif
   endif

endproc
writelib Libname.a DboxPopup.a
release procs DboxPopup.a
??"."

;==========================================================================

;call the demo
autolib = "Demolib"
ColorCal.u()
